@@ -1,5 +1,16 @@
+"""
+Module for jenkinsapi requester (which is a wrapper around python-requests)
+"""
+
 import requests
-from jenkinsapi.exceptions import JenkinsAPIException
+
+try:
+    import urlparse
+except ImportError:
+    # Python3
+    import urllib.parse as urlparse
+
+from jenkinsapi.custom_exceptions import JenkinsAPIException, PostRequired
 # import logging
 
 # # these two lines enable debugging at httplib level (requests->urllib3->httplib)
@@ -18,23 +29,24 @@ from jenkinsapi.exceptions import JenkinsAPIException
 class Requester(object):
 
     """
-    A class which carries out HTTP requests. You can replace this class with one of your own implementation if you require
-    some other way to access Jenkins.
+    A class which carries out HTTP requests. You can replace this class with one of your
+    own implementation if you require some other way to access Jenkins.
 
     This default class can handle simple authentication only.
     """
 
-    VALID_STATUS_CODES = [200,]
+    VALID_STATUS_CODES = [200, ]
 
-    def __init__(self, username=None, password=None, ssl_verify=True):
+    def __init__(self, username=None, password=None, ssl_verify=True, baseurl=None):
         if username:
             assert password, 'Cannot set a username without a password!'
 
+        self.base_scheme = urlparse.urlsplit(baseurl).scheme if baseurl else None
         self.username = username
         self.password = password
         self.ssl_verify = ssl_verify
 
-    def get_request_dict(self, url, params, data, headers):
+    def get_request_dict(self, params=None, data=None, files=None, headers=None):
         requestKwargs = {}
         if self.username:
             requestKwargs['auth'] = (self.username, self.password)
@@ -51,35 +63,55 @@ class Requester(object):
 
         requestKwargs['verify'] = self.ssl_verify
 
-        if not data == None:
+        if not data is None:
             # It may seem odd, but some Jenkins operations require posting
             # an empty string.
             requestKwargs['data'] = data
+
+        if files:
+            requestKwargs['files'] = files
+
         return requestKwargs
 
+    def _update_url_scheme(self, url):
+        """
+        Updates scheme of given url to the one used in Jenkins baseurl.
+        """
+        if self.base_scheme and not url.startswith("%s://" % self.base_scheme):
+            url_split = urlparse.urlsplit(url)
+            url = urlparse.urlunsplit(
+                [
+                    self.base_scheme,
+                    url_split.netloc,
+                    url_split.path,
+                    url_split.query,
+                    url_split.fragment
+                ]
+            )
+        return url
+
     def get_url(self, url, params=None, headers=None):
-        requestKwargs = self.get_request_dict(url, params, None, headers)
-        return requests.get(url, **requestKwargs)
+        requestKwargs = self.get_request_dict(params=params, headers=headers)
+        return requests.get(self._update_url_scheme(url), **requestKwargs)
 
-
-    def post_url(self, url, params=None, data=None, headers=None):
-        requestKwargs = self.get_request_dict(url, params, data, headers)
-        return requests.post(url, **requestKwargs)
+    def post_url(self, url, params=None, data=None, files=None, headers=None):
+        requestKwargs = self.get_request_dict(params=params, data=data, files=files, headers=headers)
+        return requests.post(self._update_url_scheme(url), **requestKwargs)
 
     def post_xml_and_confirm_status(self, url, params=None, data=None, valid=None):
         headers = {'Content-Type': 'text/xml'}
-        return self.post_and_confirm_status(url, params, data, headers, valid)
+        return self.post_and_confirm_status(url, params=params, data=data, headers=headers, valid=valid)
 
-    def post_and_confirm_status(self, url, params=None, data=None, headers=None, valid=None):
+    def post_and_confirm_status(self, url, params=None, data=None, files=None, headers=None, valid=None):
         valid = valid or self.VALID_STATUS_CODES
         assert isinstance(data, (
             str, dict)), \
             "Unexpected type of parameter 'data': %s. Expected (str, dict)" % type(data)
 
-        if not headers:
+        if not headers and not files:
             headers = {'Content-Type': 'application/x-www-form-urlencoded'}
 
-        response = self.post_url(url, params, data, headers)
+        response = self.post_url(url, params, data, files, headers)
         if not response.status_code in valid:
             raise JenkinsAPIException('Operation failed. url={0}, data={1}, headers={2}, status={3}, text={4}'.format(
                 response.url, data, headers, response.status_code, response.text.encode('UTF-8')))
@@ -89,6 +121,9 @@ class Requester(object):
         valid = valid or self.VALID_STATUS_CODES
         response = self.get_url(url, params, headers)
         if not response.status_code in valid:
-            raise JenkinsAPIException('Operation failed. url={0}, headers={1}, status={2}, text={3}'.format(
-                response.url, headers, response.status_code, response.text.encode('UTF-8')))
+            if response.status_code == 405:         # POST required
+                raise PostRequired('POST required for url {0}'.format(url))
+            else:
+                raise JenkinsAPIException('Operation failed. url={0}, headers={1}, status={2}, text={3}'.format(
+                    response.url, headers, response.status_code, response.text.encode('UTF-8')))
         return response
